@@ -1,5 +1,3 @@
-# encoding: UTF-8
-
 require "rbconfig"
 require "tempfile"
 require "stringio"
@@ -29,12 +27,12 @@ module Minitest
     def self.diff
       return @diff if defined? @diff
 
-      @diff = if (RbConfig::CONFIG["host_os"] =~ /mswin|mingw/ &&
-                  system("diff.exe", __FILE__, __FILE__)) then
+      @diff = if (RbConfig::CONFIG["host_os"] =~ /mswin|mingw/ and
+                  system "diff.exe", __FILE__, __FILE__) then
                 "diff.exe -u"
-              elsif system("gdiff", __FILE__, __FILE__)
+              elsif system "gdiff", __FILE__, __FILE__ then
                 "gdiff -u" # solaris and kin suck
-              elsif system("diff", __FILE__, __FILE__)
+              elsif system "diff", __FILE__, __FILE__ then
                 "diff -u"
               else
                 nil
@@ -59,16 +57,16 @@ module Minitest
     def diff exp, act
       result = nil
 
-      expect, butwas = things_to_diff(exp, act)
+      expect, butwas = things_to_diff exp, act
 
       return "Expected: #{mu_pp exp}\n  Actual: #{mu_pp act}" unless
         expect
 
-      Tempfile.open("expect") do |a|
+      Tempfile.create "expect" do |a|
         a.puts expect
         a.flush
 
-        Tempfile.open("butwas") do |b|
+        Tempfile.create "butwas" do |b|
           b.puts butwas
           b.flush
 
@@ -79,10 +77,10 @@ module Minitest
           if result.empty? then
             klass = exp.class
             result = [
-                      "No visible difference in the #{klass}#inspect output.\n",
-                      "You should look at the implementation of #== on ",
-                      "#{klass} or its members.\n",
-                      expect,
+                       "No visible difference in the #{klass}#inspect output.\n",
+                       "You should look at the implementation of #== on ",
+                       "#{klass} or its members.\n",
+                       expect,
                      ].join
           end
         end
@@ -127,20 +125,15 @@ module Minitest
     # See Minitest::Test.make_my_diffs_pretty!
 
     def mu_pp obj
-      s = obj.inspect
+      s = obj.inspect.encode Encoding.default_external
 
-      if defined? Encoding then
-        s = s.encode Encoding.default_external
+      return s unless String === obj &&
+        (obj.encoding != Encoding.default_external || !obj.valid_encoding?)
 
-        if String === obj && (obj.encoding != Encoding.default_external ||
-                              !obj.valid_encoding?) then
-          enc = "# encoding: #{obj.encoding}"
-          val = "#    valid: #{obj.valid_encoding?}"
-          s = "#{enc}\n#{val}\n#{s}"
-        end
-      end
+      enc = "# encoding: #{obj.encoding}"
+      val = "#    valid: #{obj.valid_encoding?}"
 
-      s
+      [enc, val, s].join "\n"
     end
 
     ##
@@ -153,8 +146,8 @@ module Minitest
       str = mu_pp obj
 
       # both '\n' & '\\n' (_after_ mu_pp (aka inspect))
-      single = !!str.match(/(?<!\\|^)\\n/)
-      double = !!str.match(/(?<=\\|^)\\n/)
+      single = str.match?(/(?<!\\|^)\\n/)
+      double = str.match?(/(?<=\\|^)\\n/)
 
       process =
         if single ^ double then
@@ -167,9 +160,9 @@ module Minitest
           :itself                                     # leave it alone
         end
 
-      str.
-        gsub(/\\?\\n/, &process).
-        gsub(/:0x[a-fA-F0-9]{4,}/m, ":0xXXXXXX") # anonymize hex values
+      str
+        .gsub(/\\?\\n/, &process)
+        .gsub(/:0x[a-fA-F0-9]{4,}/m, ":0xXXXXXX") # anonymize hex values
     end
 
     ##
@@ -194,11 +187,13 @@ module Minitest
 
     def assert_empty obj, msg = nil
       msg = message(msg) { "Expected #{mu_pp(obj)} to be empty" }
-      assert_respond_to obj, :empty?
-      assert obj.empty?, msg
+      assert_predicate obj, :empty?, msg
     end
 
-    E = "" # :nodoc:
+    def _where # :nodoc:
+      Minitest.filter_backtrace(caller).first
+        .split(":in ", 2).first # clean up noise
+    end
 
     ##
     # Fails unless <tt>exp == act</tt> printing the difference between
@@ -214,21 +209,11 @@ module Minitest
     # See also: Minitest::Assertions.diff
 
     def assert_equal exp, act, msg = nil
-      msg = message(msg, E) { diff exp, act }
-      result = assert exp == act, msg
+      msg = message(msg, nil) { diff exp, act }
 
-      if nil == exp then
-        if Minitest::VERSION =~ /^6/ then
-          refute_nil exp, "Use assert_nil if expecting nil."
-        else
-          where = Minitest.filter_backtrace(caller).first
-          where = where.split(/:in /, 2).first # clean up noise
+      refute_nil exp, message { "Use assert_nil if expecting nil" } if exp.nil? # don't count
 
-          warn "DEPRECATED: Use assert_nil if expecting nil from #{where}. This will fail in Minitest 6."
-        end
-      end
-
-      result
+      assert exp == act, msg
     end
 
     ##
@@ -258,10 +243,9 @@ module Minitest
 
     def assert_includes collection, obj, msg = nil
       msg = message(msg) {
-        "Expected #{mu_pp(collection)} to include #{mu_pp(obj)}"
+        "Expected #{mu_pp collection} to include #{mu_pp obj}"
       }
-      assert_respond_to collection, :include?
-      assert collection.include?(obj), msg
+      assert_operator collection, :include?, obj, msg
     end
 
     ##
@@ -269,7 +253,7 @@ module Minitest
 
     def assert_instance_of cls, obj, msg = nil
       msg = message(msg) {
-        "Expected #{mu_pp(obj)} to be an instance of #{cls}, not #{obj.class}"
+        "Expected #{mu_pp obj} to be an instance of #{cls}, not #{obj.class}"
       }
 
       assert obj.instance_of?(cls), msg
@@ -280,7 +264,8 @@ module Minitest
 
     def assert_kind_of cls, obj, msg = nil
       msg = message(msg) {
-        "Expected #{mu_pp(obj)} to be a kind of #{cls}, not #{obj.class}" }
+        "Expected #{mu_pp obj} to be a kind of #{cls}, not #{obj.class}"
+      }
 
       assert obj.kind_of?(cls), msg
     end
@@ -290,7 +275,7 @@ module Minitest
 
     def assert_match matcher, obj, msg = nil
       msg = message(msg) { "Expected #{mu_pp matcher} to match #{mu_pp obj}" }
-      assert_respond_to matcher, :"=~"
+      assert_respond_to matcher, :=~
       matcher = Regexp.new Regexp.escape matcher if String === matcher
       assert matcher =~ obj, msg
 
@@ -301,7 +286,7 @@ module Minitest
     # Fails unless +obj+ is nil
 
     def assert_nil obj, msg = nil
-      msg = message(msg) { "Expected #{mu_pp(obj)} to be nil" }
+      msg = message(msg) { "Expected #{mu_pp obj} to be nil" }
       assert obj.nil?, msg
     end
 
@@ -312,7 +297,8 @@ module Minitest
 
     def assert_operator o1, op, o2 = UNDEFINED, msg = nil
       return assert_predicate o1, op, msg if UNDEFINED == o2
-      msg = message(msg) { "Expected #{mu_pp(o1)} to be #{op} #{mu_pp(o2)}" }
+      assert_respond_to o1, op
+      msg = message(msg) { "Expected #{mu_pp o1} to be #{op} #{mu_pp o2}" }
       assert o1.__send__(op, o2), msg
     end
 
@@ -372,15 +358,12 @@ module Minitest
     # error.
 
     def assert_pattern
-      raise NotImplementedError, "only available in Ruby 3.0+" unless RUBY_VERSION >= "3.0"
       flunk "assert_pattern requires a block to capture errors." unless block_given?
 
-      begin # TODO: remove after ruby 2.6 dropped
-        yield
-        pass
-      rescue NoMatchingPatternError => e
-        flunk e.message
-      end
+      yield
+      pass
+    rescue NoMatchingPatternError => e
+      flunk e.message
     end
 
     ##
@@ -393,7 +376,8 @@ module Minitest
     #   str.must_be :empty?
 
     def assert_predicate o1, op, msg = nil
-      msg = message(msg) { "Expected #{mu_pp(o1)} to be #{op}" }
+      assert_respond_to o1, op, include_all:true
+      msg = message(msg) { "Expected #{mu_pp o1} to be #{op}" }
       assert o1.__send__(op), msg
     end
 
@@ -438,23 +422,22 @@ module Minitest
         raise
       rescue Exception => e
         flunk proc {
-          exception_details(e, "#{msg}#{mu_pp(exp)} exception expected, not")
+          exception_details(e, "#{msg}#{mu_pp exp} exception expected, not")
         }
       end
 
       exp = exp.first if exp.size == 1
 
-      flunk "#{msg}#{mu_pp(exp)} expected but nothing was raised."
+      flunk "#{msg}#{mu_pp exp} expected but nothing was raised."
     end
 
     ##
     # Fails unless +obj+ responds to +meth+.
+    # include_all defaults to false to match Object#respond_to?
 
-    def assert_respond_to obj, meth, msg = nil
-      msg = message(msg) {
-        "Expected #{mu_pp(obj)} (#{obj.class}) to respond to ##{meth}"
-      }
-      assert obj.respond_to?(meth), msg
+    def assert_respond_to obj, meth, msg = nil, include_all: false
+      msg = message(msg) { "Expected #{mu_pp obj} (#{obj.class}) to respond to ##{meth}" }
+      assert obj.respond_to?(meth, include_all), msg
     end
 
     ##
@@ -468,22 +451,6 @@ module Minitest
       assert exp.equal?(act), msg
     end
 
-    ##
-    # +send_ary+ is a receiver, message and arguments.
-    #
-    # Fails unless the call returns a true value
-
-    def assert_send send_ary, m = nil
-      where = Minitest.filter_backtrace(caller).first
-      where = where.split(/:in /, 2).first # clean up noise
-      warn "DEPRECATED: assert_send. From #{where}"
-
-      recv, msg, *args = send_ary
-      m = message(m) {
-        "Expected #{mu_pp(recv)}.#{msg}(*#{mu_pp(args)}) to return true" }
-      assert recv.__send__(msg, *args), m
-    end
-
     ##
     # Fails if the block outputs anything to stderr or stdout.
     #
@@ -499,19 +466,14 @@ module Minitest
     # Fails unless the block throws +sym+
 
     def assert_throws sym, msg = nil
-      default = "Expected #{mu_pp(sym)} to have been thrown"
+      default = "Expected #{mu_pp sym} to have been thrown"
       caught = true
-      value = catch(sym) do
+      value = catch sym do
         begin
           yield
-        rescue ThreadError => e       # wtf?!? 1.8 + threads == suck
-          default += ", not \:#{e.message[/uncaught throw \`(\w+?)\'/, 1]}"
-        rescue ArgumentError => e     # 1.9 exception
-          raise e unless e.message.include?("uncaught throw")
+        rescue ArgumentError => e     # 1.9+ exception
+          raise e unless e.message.include? "uncaught throw"
           default += ", not #{e.message.split(/ /).last}"
-        rescue NameError => e         # 1.8 exception
-          raise e unless e.name == sym
-          default += ", not #{e.name.inspect}"
         end
         caught = false
       end
@@ -606,12 +568,12 @@ module Minitest
 
     def exception_details e, msg
       [
-       "#{msg}",
-       "Class: <#{e.class}>",
-       "Message: <#{e.message.inspect}>",
-       "---Backtrace---",
-       "#{Minitest.filter_backtrace(e.backtrace).join("\n")}",
-       "---------------",
+        msg,
+        "Class: <#{e.class}>",
+        "Message: <#{e.message.inspect}>",
+        "---Backtrace---",
+        Minitest.filter_backtrace(e.backtrace),
+        "---------------",
       ].join "\n"
     end
 
@@ -620,7 +582,7 @@ module Minitest
     # you to put time-bombs in your tests if you need to keep
     # something around until a later date lest you forget about it.
 
-    def fail_after y,m,d,msg
+    def fail_after y, m, d, msg
       flunk msg if Time.now > Time.local(y, m, d)
     end
 
@@ -633,13 +595,16 @@ module Minitest
     end
 
     ##
-    # Returns a proc that will output +msg+ along with the default message.
+    # Returns a proc that delays generation of an output message. If
+    # +msg+ is a proc (eg, from another +message+ call) return +msg+
+    # as-is. Otherwise, return a proc that will output +msg+ along
+    # with the value of the result of the block passed to +message+.
 
-    def message msg = nil, ending = nil, &default
+    def message msg = nil, ending = ".", &default
+      return msg if Proc === msg
       proc {
-        msg = msg.call.chomp(".") if Proc === msg
         custom_message = "#{msg}.\n" unless msg.nil? or msg.to_s.empty?
-        "#{custom_message}#{default.call}#{ending || "."}"
+        "#{custom_message}#{default.call}#{ending}"
       }
     end
 
@@ -654,7 +619,7 @@ module Minitest
     # Fails if +test+ is truthy.
 
     def refute test, msg = nil
-      msg ||= message { "Expected #{mu_pp(test)} to not be truthy" }
+      msg ||= message { "Expected #{mu_pp test} to not be truthy" }
       assert !test, msg
     end
 
@@ -662,9 +627,8 @@ module Minitest
     # Fails if +obj+ is empty.
 
     def refute_empty obj, msg = nil
-      msg = message(msg) { "Expected #{mu_pp(obj)} to not be empty" }
-      assert_respond_to obj, :empty?
-      refute obj.empty?, msg
+      msg = message(msg) { "Expected #{mu_pp obj} to not be empty" }
+      refute_predicate obj, :empty?, msg
     end
 
     ##
@@ -674,7 +638,7 @@ module Minitest
 
     def refute_equal exp, act, msg = nil
       msg = message(msg) {
-        "Expected #{mu_pp(act)} to not be equal to #{mu_pp(exp)}"
+        "Expected #{mu_pp act} to not be equal to #{mu_pp exp}"
       }
       refute exp == act, msg
     end
@@ -696,19 +660,16 @@ module Minitest
     # For comparing Floats.  Fails if +exp+ and +act+ have a relative error
     # less than +epsilon+.
 
-    def refute_in_epsilon a, b, epsilon = 0.001, msg = nil
-      refute_in_delta a, b, a * epsilon, msg
+    def refute_in_epsilon exp, act, epsilon = 0.001, msg = nil
+      refute_in_delta exp, act, [exp.abs, act.abs].min * epsilon, msg
     end
 
     ##
-    # Fails if +collection+ includes +obj+.
+    # Fails if +obj+ includes +sub+.
 
-    def refute_includes collection, obj, msg = nil
-      msg = message(msg) {
-        "Expected #{mu_pp(collection)} to not include #{mu_pp(obj)}"
-      }
-      assert_respond_to collection, :include?
-      refute collection.include?(obj), msg
+    def refute_includes obj, sub, msg = nil
+      msg = message(msg) { "Expected #{mu_pp obj} to not include #{mu_pp sub}" }
+      refute_operator obj, :include?, sub, msg
     end
 
     ##
@@ -716,7 +677,7 @@ module Minitest
 
     def refute_instance_of cls, obj, msg = nil
       msg = message(msg) {
-        "Expected #{mu_pp(obj)} to not be an instance of #{cls}"
+        "Expected #{mu_pp obj} to not be an instance of #{cls}"
       }
       refute obj.instance_of?(cls), msg
     end
@@ -725,7 +686,7 @@ module Minitest
     # Fails if +obj+ is a kind of +cls+.
 
     def refute_kind_of cls, obj, msg = nil
-      msg = message(msg) { "Expected #{mu_pp(obj)} to not be a kind of #{cls}" }
+      msg = message(msg) { "Expected #{mu_pp obj} to not be a kind of #{cls}" }
       refute obj.kind_of?(cls), msg
     end
 
@@ -734,16 +695,15 @@ module Minitest
 
     def refute_match matcher, obj, msg = nil
       msg = message(msg) { "Expected #{mu_pp matcher} to not match #{mu_pp obj}" }
-      assert_respond_to matcher, :"=~"
       matcher = Regexp.new Regexp.escape matcher if String === matcher
-      refute matcher =~ obj, msg
+      refute_operator matcher, :=~, obj, msg
     end
 
     ##
     # Fails if +obj+ is nil.
 
     def refute_nil obj, msg = nil
-      msg = message(msg) { "Expected #{mu_pp(obj)} to not be nil" }
+      msg = message(msg) { "Expected #{mu_pp obj} to not be nil" }
       refute obj.nil?, msg
     end
 
@@ -760,15 +720,12 @@ module Minitest
     # other exceptions will be raised as normal and generate a test error.
 
     def refute_pattern
-      raise NotImplementedError, "only available in Ruby 3.0+" unless RUBY_VERSION >= "3.0"
       flunk "refute_pattern requires a block to capture errors." unless block_given?
 
-      begin
-        yield
-        flunk("NoMatchingPatternError expected, but nothing was raised.")
-      rescue NoMatchingPatternError
-        pass
-      end
+      yield
+      flunk "NoMatchingPatternError expected, but nothing was raised."
+    rescue NoMatchingPatternError
+      pass
     end
 
     ##
@@ -779,7 +736,8 @@ module Minitest
 
     def refute_operator o1, op, o2 = UNDEFINED, msg = nil
       return refute_predicate o1, op, msg if UNDEFINED == o2
-      msg = message(msg) { "Expected #{mu_pp(o1)} to not be #{op} #{mu_pp(o2)}" }
+      assert_respond_to o1, op
+      msg = message(msg) { "Expected #{mu_pp o1} to not be #{op} #{mu_pp o2}" }
       refute o1.__send__(op, o2), msg
     end
 
@@ -801,17 +759,19 @@ module Minitest
     #   str.wont_be :empty?
 
     def refute_predicate o1, op, msg = nil
-      msg = message(msg) { "Expected #{mu_pp(o1)} to not be #{op}" }
+      assert_respond_to o1, op
+      msg = message(msg) { "Expected #{mu_pp o1} to not be #{op}" }
       refute o1.__send__(op), msg
     end
 
     ##
     # Fails if +obj+ responds to the message +meth+.
+    # include_all defaults to false to match Object#respond_to?
 
-    def refute_respond_to obj, meth, msg = nil
-      msg = message(msg) { "Expected #{mu_pp(obj)} to not respond to #{meth}" }
+    def refute_respond_to obj, meth, msg = nil, include_all: false
+      msg = message(msg) { "Expected #{mu_pp obj} to not respond to #{meth}" }
 
-      refute obj.respond_to?(meth), msg
+      refute obj.respond_to?(meth, include_all), msg
     end
 
     ##
@@ -830,10 +790,10 @@ module Minitest
     # gets listed at the end of the run but doesn't cause a failure
     # exit code.
 
-    def skip msg = nil, bt = caller
+    def skip msg = nil, _ignored = nil
       msg ||= "Skipped, no message given"
       @skip = true
-      raise Minitest::Skip, msg, bt
+      raise Minitest::Skip, msg
     end
 
     ##
@@ -842,9 +802,9 @@ module Minitest
     # date, but still holds you accountable and prevents you from
     # forgetting it.
 
-    def skip_until y,m,d,msg
+    def skip_until y, m, d, msg
       skip msg if Time.now < Time.local(y, m, d)
-      where = caller.first.rpartition(':in').reject(&:empty?).first
+      where = caller(1..1).first.rpartition(":in").reject(&:empty?).first
       warn "Stale skip_until %p at %s" % [msg, where]
     end
 
