@@ -38,12 +38,14 @@ module EnumerableSpecs
   class Empty
     include Enumerable
     def each
+      self
     end
   end
 
   class EmptyWithSize
     include Enumerable
     def each
+      self
     end
     def size
       0
@@ -343,9 +345,6 @@ module EnumerableSpecs
     end
   end
 
-  # Set is a core class since Ruby 3.2
-  ruby_version_is '3.2' do
-    class SetSubclass < Set
-    end
+  class SetSubclass < Set
   end
 end # EnumerableSpecs utility classes
