@@ -13,11 +13,9 @@ describe "ENV#clone" do
     }.should raise_error(ArgumentError)
   end
 
-  ruby_version_is "3.2" do
-    it "raises TypeError" do
-      -> {
-        ENV.clone
-      }.should raise_error(TypeError, /Cannot clone ENV, use ENV.to_h to get a copy of ENV as a hash/)
-    end
+  it "raises TypeError" do
+    -> {
+      ENV.clone
+    }.should raise_error(TypeError, /Cannot clone ENV, use ENV.to_h to get a copy of ENV as a hash/)
   end
 end
