@@ -1,4 +1,4 @@
-# -*- encoding: binary -*-
+# encoding: binary
 require_relative '../../../spec_helper'
 
 describe :range_cover_and_include, shared: true do
@@ -19,6 +19,27 @@ describe :range_cover_and_include, shared: true do
     (0.5...2.4).send(@method, 2.4).should == false
   end
 
+  it "returns true if other is an element of self for endless ranges" do
+    (1..).send(@method, 2.4).should == true
+    (0.5...).send(@method, 2.4).should == true
+  end
+
+  it "returns true if other is an element of self for beginless ranges" do
+    (..10).send(@method, 2.4).should == true
+    (...10.5).send(@method, 2.4).should == true
+  end
+
+  it "returns false if values are not comparable" do
+    (1..10).send(@method, nil).should == false
+    (1...10).send(@method, nil).should == false
+
+    (..10).send(@method, nil).should == false
+    (...10).send(@method, nil).should == false
+
+    (1..).send(@method, nil).should == false
+    (1...).send(@method, nil).should == false
+  end
+
   it "compares values using <=>" do
     rng = (1..5)
     m = mock("int")
@@ -47,7 +68,6 @@ describe :range_cover_and_include, shared: true do
   it "returns true if argument is less than the last value of the range and greater than the first value" do
     (20..30).send(@method, 28).should be_true
     ('e'..'h').send(@method, 'g').should be_true
-    ("\u{999}".."\u{9999}").send @method, "\u{9995}"
   end
 
   it "returns true if argument is sole element in the range" do
