@@ -69,22 +69,18 @@ describe "Time#zone" do
     Time.at(Time.now, in: 'UTC').zone.should == "UTC"
     Time.at(Time.now, in: 'Z').zone.should == "UTC"
 
-    ruby_version_is "3.1" do
-      Time.new(2022, 1, 1, 0, 0, 0, "-00:00").zone.should == "UTC"
-      Time.now.localtime("-00:00").zone.should == "UTC"
-      Time.at(Time.now, in: '-00:00').zone.should == "UTC"
-    end
+    Time.new(2022, 1, 1, 0, 0, 0, "-00:00").zone.should == "UTC"
+    Time.now.localtime("-00:00").zone.should == "UTC"
+    Time.at(Time.now, in: '-00:00').zone.should == "UTC"
 
-    ruby_version_is "3.1" do
-      Time.new(2022, 1, 1, 0, 0, 0, in: "UTC").zone.should == "UTC"
-      Time.new(2022, 1, 1, 0, 0, 0, in: "Z").zone.should == "UTC"
+    Time.new(2022, 1, 1, 0, 0, 0, in: "UTC").zone.should == "UTC"
+    Time.new(2022, 1, 1, 0, 0, 0, in: "Z").zone.should == "UTC"
 
-      Time.now(in: 'UTC').zone.should == "UTC"
-      Time.now(in: 'Z').zone.should == "UTC"
+    Time.now(in: 'UTC').zone.should == "UTC"
+    Time.now(in: 'Z').zone.should == "UTC"
 
-      Time.at(Time.now, in: 'UTC').zone.should == "UTC"
-      Time.at(Time.now, in: 'Z').zone.should == "UTC"
-    end
+    Time.at(Time.now, in: 'UTC').zone.should == "UTC"
+    Time.at(Time.now, in: 'Z').zone.should == "UTC"
   end
 
   platform_is_not :aix, :windows do
