@@ -33,19 +33,9 @@ describe "DateTime#strftime" do
     @time.strftime("%Z").should == "+00:00"
   end
 
-  # %v is %e-%b-%Y for Date/DateTime
-  version_is date_version, ""..."3.2" do #ruby_version_is ""..."3.1" do
-    it "should be able to show the commercial week" do
-      @time.strftime("%v").should == " 3-Feb-2001"
-      @time.strftime("%v").should == @time.strftime('%e-%b-%Y')
-    end
-  end
-
-  version_is date_version, "3.2" do #ruby_version_is "3.1" do
-    it "should be able to show the commercial week" do
-      @time.strftime("%v").should == " 3-FEB-2001"
-      @time.strftime("%v").should != @time.strftime('%e-%b-%Y')
-    end
+  it "should be able to show the commercial week" do
+    @time.strftime("%v").should == " 3-FEB-2001"
+    @time.strftime("%v").should != @time.strftime('%e-%b-%Y')
   end
 
   # additional conversion specifiers only in Date/DateTime
