@@ -262,6 +262,7 @@ class TestData < Test::Unit::TestCase
     assert_equal('#<data >', test.inspect)
     assert_equal([], test.members)
     assert_equal({}, test.to_h)
+    assert_predicate(test, :frozen?)
   end
 
   def test_dup
@@ -280,4 +281,10 @@ class TestData < Test::Unit::TestCase
     assert_not_same(test, loaded)
     assert_predicate(loaded, :frozen?)
   end
+
+  def test_frozen_subclass
+    test = Class.new(Data.define(:a)).freeze.new(a: 0)
+    assert_kind_of(Data, test)
+    assert_equal([:a], test.members)
+  end
 end
