@@ -5,6 +5,7 @@ require_relative "multifactor_auth_utilities"
 require "rubygems"
 require "rubygems/command"
 require "rubygems/gemcutter_utilities"
+require "rubygems/config_file"
 
 class TestGemGemcutterUtilities < Gem::TestCase
   def setup
@@ -41,7 +42,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
     }
 
     File.open Gem.configuration.credentials_path, "w" do |f|
-      f.write keys.to_yaml
+      f.write Gem::ConfigFile.dump_with_rubygems_yaml(keys)
     end
 
     ENV["RUBYGEMS_HOST"] = "http://rubygems.engineyard.com"
@@ -52,10 +53,10 @@ class TestGemGemcutterUtilities < Gem::TestCase
   end
 
   def test_api_key
-    keys = { :rubygems_api_key => "KEY" }
+    keys = { rubygems_api_key: "KEY" }
 
     File.open Gem.configuration.credentials_path, "w" do |f|
-      f.write keys.to_yaml
+      f.write Gem::ConfigFile.dump_with_rubygems_yaml(keys)
     end
 
     Gem.configuration.load_api_keys
@@ -64,10 +65,10 @@ class TestGemGemcutterUtilities < Gem::TestCase
   end
 
   def test_api_key_override
-    keys = { :rubygems_api_key => "KEY", :other => "OTHER" }
+    keys = { rubygems_api_key: "KEY", other: "OTHER" }
 
     File.open Gem.configuration.credentials_path, "w" do |f|
-      f.write keys.to_yaml
+      f.write Gem::ConfigFile.dump_with_rubygems_yaml(keys)
     end
 
     Gem.configuration.load_api_keys
@@ -97,9 +98,9 @@ class TestGemGemcutterUtilities < Gem::TestCase
   def test_sign_in
     util_sign_in
 
-    assert_match %r{Enter your RubyGems.org credentials.}, @sign_in_ui.output
+    assert_match(/Enter your RubyGems.org credentials./, @sign_in_ui.output)
     assert @fetcher.last_request["authorization"]
-    assert_match %r{Signed in.}, @sign_in_ui.output
+    assert_match(/Signed in./, @sign_in_ui.output)
 
     credentials = load_yaml_file Gem.configuration.credentials_path
     assert_equal @fetcher.api_key, credentials[:rubygems_api_key]
@@ -112,7 +113,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
     assert_match "Enter your http://example.com credentials.",
                  @sign_in_ui.output
     assert @fetcher.last_request["authorization"]
-    assert_match %r{Signed in.}, @sign_in_ui.output
+    assert_match(/Signed in./, @sign_in_ui.output)
 
     credentials = load_yaml_file Gem.configuration.credentials_path
     assert_equal @fetcher.api_key, credentials["http://example.com"]
@@ -125,7 +126,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
     assert_match "Enter your RubyGems.org credentials.",
                  @sign_in_ui.output
     assert @fetcher.last_request["authorization"]
-    assert_match %r{Signed in.}, @sign_in_ui.output
+    assert_match(/Signed in./, @sign_in_ui.output)
 
     credentials = load_yaml_file Gem.configuration.credentials_path
     assert_equal @fetcher.api_key, credentials[:rubygems_api_key]
@@ -138,7 +139,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
     assert_match "Enter your http://example.com credentials.",
                  @sign_in_ui.output
     assert @fetcher.last_request["authorization"]
-    assert_match %r{Signed in.}, @sign_in_ui.output
+    assert_match(/Signed in./, @sign_in_ui.output)
 
     credentials = load_yaml_file Gem.configuration.credentials_path
     assert_equal @fetcher.api_key, credentials["http://example.com"]
@@ -163,13 +164,15 @@ class TestGemGemcutterUtilities < Gem::TestCase
   def test_sign_in_with_other_credentials_doesnt_overwrite_other_keys
     other_api_key = "f46dbb18bb6a9c97cdc61b5b85c186a17403cdcbf"
 
+    config = Hash[:other_api_key, other_api_key]
+
     File.open Gem.configuration.credentials_path, "w" do |f|
-      f.write Hash[:other_api_key, other_api_key].to_yaml
+      f.write Gem::ConfigFile.dump_with_rubygems_yaml(config)
     end
     util_sign_in
 
-    assert_match %r{Enter your RubyGems.org credentials.}, @sign_in_ui.output
-    assert_match %r{Signed in.}, @sign_in_ui.output
+    assert_match(/Enter your RubyGems.org credentials./, @sign_in_ui.output)
+    assert_match(/Signed in./, @sign_in_ui.output)
 
     credentials = load_yaml_file Gem.configuration.credentials_path
     assert_equal @fetcher.api_key, credentials[:rubygems_api_key]
@@ -182,8 +185,8 @@ class TestGemGemcutterUtilities < Gem::TestCase
       util_sign_in
     end
 
-    assert_match %r{Enter your RubyGems.org credentials.}, @sign_in_ui.output
-    assert_match %r{Access Denied.}, @sign_in_ui.output
+    assert_match(/Enter your RubyGems.org credentials./, @sign_in_ui.output)
+    assert_match(/Access Denied./, @sign_in_ui.output)
   end
 
   def test_signin_with_env_otp_code
@@ -230,9 +233,10 @@ class TestGemGemcutterUtilities < Gem::TestCase
       end
     end
 
-    assert_match "You have enabled multi-factor authentication. Please visit #{@fetcher.webauthn_url_with_port(server.port)} " \
+    assert_match "You have enabled multi-factor authentication. Please visit the following URL " \
       "to authenticate via security device. If you can't verify using WebAuthn but have OTP enabled, " \
       "you can re-run the gem signin command with the `--otp [your_code]` option.", @sign_in_ui.output
+    assert_match @fetcher.webauthn_url_with_port(server.port), @sign_in_ui.output
     assert_match "You are verified with a security device. You may close the browser window.", @sign_in_ui.output
     assert_equal "Uvh6T57tkWuUnWYo", @fetcher.last_request["OTP"]
   end
@@ -252,9 +256,10 @@ class TestGemGemcutterUtilities < Gem::TestCase
     end
     assert_equal 1, error.exit_code
 
-    assert_match "You have enabled multi-factor authentication. Please visit #{@fetcher.webauthn_url_with_port(server.port)} " \
+    assert_match "You have enabled multi-factor authentication. Please visit the following URL " \
       "to authenticate via security device. If you can't verify using WebAuthn but have OTP enabled, " \
       "you can re-run the gem signin command with the `--otp [your_code]` option.", @sign_in_ui.output
+    assert_match @fetcher.webauthn_url_with_port(server.port), @sign_in_ui.output
     assert_match "ERROR:  Security device verification failed: Something went wrong", @sign_in_ui.error
     refute_match "You are verified with a security device. You may close the browser window.", @sign_in_ui.output
     refute_match "Signed in with API key:", @sign_in_ui.output
@@ -270,9 +275,10 @@ class TestGemGemcutterUtilities < Gem::TestCase
       util_sign_in
     end
 
-    assert_match "You have enabled multi-factor authentication. Please visit #{@fetcher.webauthn_url_with_port(server.port)} " \
+    assert_match "You have enabled multi-factor authentication. Please visit the following URL " \
       "to authenticate via security device. If you can't verify using WebAuthn but have OTP enabled, " \
       "you can re-run the gem signin command with the `--otp [your_code]` option.", @sign_in_ui.output
+    assert_match @fetcher.webauthn_url_with_port(server.port), @sign_in_ui.output
     assert_match "You are verified with a security device. You may close the browser window.", @sign_in_ui.output
     assert_equal "Uvh6T57tkWuUnWYo", @fetcher.last_request["OTP"]
   end
@@ -289,9 +295,10 @@ class TestGemGemcutterUtilities < Gem::TestCase
       end
     end
 
-    assert_match "You have enabled multi-factor authentication. Please visit #{@fetcher.webauthn_url_with_port(server.port)} " \
+    assert_match "You have enabled multi-factor authentication. Please visit the following URL " \
       "to authenticate via security device. If you can't verify using WebAuthn but have OTP enabled, " \
       "you can re-run the gem signin command with the `--otp [your_code]` option.", @sign_in_ui.output
+    assert_match @fetcher.webauthn_url_with_port(server.port), @sign_in_ui.output
     assert_match "ERROR:  Security device verification failed: " \
       "The token in the link you used has either expired or been used already.", @sign_in_ui.error
   end
@@ -303,7 +310,7 @@ class TestGemGemcutterUtilities < Gem::TestCase
     ENV["RUBYGEMS_HOST"] = @fetcher.host
     Gem::RemoteFetcher.fetcher = @fetcher
 
-    @sign_in_ui = Gem::MockGemUi.new("#{email}\n#{password}\n\n\n\n\n\n\n\n\n" + extra_input)
+    @sign_in_ui = Gem::MockGemUi.new("#{email}\n#{password}\n\n\n" + extra_input)
 
     use_ui @sign_in_ui do
       if args.length > 0
@@ -315,9 +322,9 @@ class TestGemGemcutterUtilities < Gem::TestCase
   end
 
   def test_verify_api_key
-    keys = { :other => "a5fdbb6ba150cbb83aad2bb2fede64cf040453903" }
+    keys = { other: "a5fdbb6ba150cbb83aad2bb2fede64cf040453903" }
     File.open Gem.configuration.credentials_path, "w" do |f|
-      f.write keys.to_yaml
+      f.write Gem::ConfigFile.dump_with_rubygems_yaml(keys)
     end
     Gem.configuration.load_api_keys
 
