@@ -12,6 +12,14 @@ class TestGemRequirement < Gem::TestCase
     assert_equal [[">=", v(1)], ["<", v(2)]], r.requirements
   end
 
+  def test_initialize_copy
+    r = req("= 1.2")
+    r2 = r.dup
+
+    assert_equal r.requirements, r2.requirements
+    refute_same r.requirements, r2.requirements
+  end
+
   def test_equals2
     r = req "= 1.2"
     assert_equal r, r.dup
@@ -84,7 +92,7 @@ class TestGemRequirement < Gem::TestCase
       Gem::Requirement.parse(Gem::Version.new("2"))
   end
 
-  if !(Gem.java_platform? && ENV["JRUBY_OPTS"].to_s.include?("--debug"))
+  unless Gem.java_platform? && ENV["JRUBY_OPTS"].to_s.include?("--debug")
     def test_parse_deduplication
       assert_same "~>", Gem::Requirement.parse("~> 1").first
     end
@@ -129,11 +137,7 @@ class TestGemRequirement < Gem::TestCase
     refute_satisfied_by "1.2", r
     assert_satisfied_by "1.3", r
 
-    assert_raise ArgumentError do
-      Gem::Deprecate.skip_during do
-        assert_satisfied_by nil, r
-      end
-    end
+    assert_satisfied_by nil, r
   end
 
   def test_satisfied_by_eh_blank
@@ -143,11 +147,7 @@ class TestGemRequirement < Gem::TestCase
     assert_satisfied_by "1.2", r
     refute_satisfied_by "1.3", r
 
-    assert_raise ArgumentError do
-      Gem::Deprecate.skip_during do
-        assert_satisfied_by nil, r
-      end
-    end
+    refute_satisfied_by nil, r
   end
 
   def test_satisfied_by_eh_equal
@@ -157,11 +157,7 @@ class TestGemRequirement < Gem::TestCase
     assert_satisfied_by "1.2", r
     refute_satisfied_by "1.3", r
 
-    assert_raise ArgumentError do
-      Gem::Deprecate.skip_during do
-        assert_satisfied_by nil, r
-      end
-    end
+    refute_satisfied_by nil, r
   end
 
   def test_satisfied_by_eh_gt
@@ -171,9 +167,7 @@ class TestGemRequirement < Gem::TestCase
     refute_satisfied_by "1.2", r
     assert_satisfied_by "1.3", r
 
-    assert_raise ArgumentError do
-      r.satisfied_by? nil
-    end
+    refute_satisfied_by nil, r
   end
 
   def test_satisfied_by_eh_gte
@@ -287,7 +281,7 @@ class TestGemRequirement < Gem::TestCase
   end
 
   def test_illformed_requirements
-    [ ">>> 1.3.5", "> blah" ].each do |rq|
+    [">>> 1.3.5", "> blah"].each do |rq|
       assert_raise Gem::Requirement::BadRequirementError, "req [#{rq}] should fail" do
         Gem::Requirement.new rq
       end
@@ -437,19 +431,19 @@ class TestGemRequirement < Gem::TestCase
   end
 
   def test_marshal_load_attack
-    wa = Net::WriteAdapter.allocate
+    wa = Gem::Net::WriteAdapter.allocate
     wa.instance_variable_set(:@socket, self.class)
     wa.instance_variable_set(:@method_id, :exploit)
     request_set = Gem::RequestSet.allocate
     request_set.instance_variable_set(:@git_set, "id")
     request_set.instance_variable_set(:@sets, wa)
-    wa = Net::WriteAdapter.allocate
+    wa = Gem::Net::WriteAdapter.allocate
     wa.instance_variable_set(:@socket, request_set)
     wa.instance_variable_set(:@method_id, :resolve)
     ent = Gem::Package::TarReader::Entry.allocate
     ent.instance_variable_set(:@read, 0)
     ent.instance_variable_set(:@header, "aaa")
-    io = Net::BufferedIO.allocate
+    io = Gem::Net::BufferedIO.allocate
     io.instance_variable_set(:@io, ent)
     io.instance_variable_set(:@debug_output, wa)
     reader = Gem::Package::TarReader.allocate
